#!/bin/bash
#
# Runs multiple realizations of a set of experiments (defined in
# the config.yml)
#
# How to use this script:
#
# 1) clone the autosubmit-api repo, github.com:BSC-ES/autosubmit-api.git
# 2) install autosubmit in a virtual environment  (using venv)
#
# (nohup) ./run_multiple_realizations.sh hyperqueue 30
# to run 30 times the testing suite of hyperqueue
#
# This script will create status for each realization so that the
# user is aware of how the experiments finished.
#
# Copyright (c) 2025  Manuel G. Marciani
# BSC-CNS - Earth Sciences
#

# loads the testing suite environment from t
function load_testing_suite() {
    # load the testing_suite environment
    echo -n "testing_suite_wrapper.sh: Info: Loading micromamba... "    
    MAMBA_ROOT_PREFIX=${HOME}/Documents/local/dev/venvs
    MICROMAMBAPATH=${HOME}/Documents/local/dev/micromamba
    export PATH=${MICROMAMBAPATH}/:$PATH
    if [[ -n "$(micromamba 2>&1 /dev/null)" ]]; then 
        echo "Loaded!"
    else
        echo ""
        echo "testing_suite_wrapper.sh: Error: Could not load Micromamba from $MICROMAMBAPATH"    
    fi
    eval "$(micromamba shell hook --shell zsh)"

    echo -n "testing_suite_wrapper.sh: Info: Loading the Testing suite..."    
    
    TESTING_SUITE_PATH=/home/mgimenez/Documents/local/dev/venvs/testing_suite
    micromamba activate $TESTING_SUITE_PATH 

    if [[ -n "$(testing_suite 2>&1 /dev/null)" ]]; then 
        echo "Loaded!"
    else
        echo ""
        echo "testing_suite_wrapper.sh: Error: Could not load the testing_suite from $TESTING_SUITE_PATH"    
    fi

}

function load_autosubmit_api() {
    # start the api docker image
    API_PORT=8099 #this is hardcoded on t
    AUTOSUBMIT_DATABASE=/home/mgimenez/autosubmit
    echo -n "testing_suite_wrapper.sh: Info: Starting the API listening at $API_PORT with databases at $AUTOSUBMIT_DATABASE.."    
    docker run --name autosubmit-api-container \
        --rm -d -p $API_PORT:8000 \
        --mount type=bind,src=$AUTOSUBMIT_DATABASE,dst=/app/autosubmit/database,bind-propagation=shared \
        --mount type=bind,src=$AUTOSUBMIT_DATABASE,dst=/app/autosubmit/experiments,bind-propagation=shared \
        -e "PROTECTION_LEVEL=NONE" \
        autosubmit-api > /dev/null

    # sleep until the api responds 
    while [[ -z "$(curl -f localhost:$API_PORT 2> /dev/null | grep Autosubmit)" ]]; do
        sleep 0.1
        echo -n "."
    done 
    echo "API started!"    
}

function stop_autosubmit_api() {
    echo -n "testing_suite_wrapper.sh: Info: Stopping api..."    
    if [[ "$(docker kill autosubmit-api-container 2>&1)" == "autosubmit-api-container" ]]; then
        echo "Stopped!"    
    else 
        echo ""
        echo "testing_suite_wrapper.sh: Error: could not stop the api"    
    fi
}

function load_autosubmit() {
    echo -n "testing_suite_wrapper.sh: Info: Loading Autosubmit..."    
    # load the autosubmit environment
    AUTOSUBMIT_VENV=/home/mgimenez/Documents/local/dev/venvs/autosubmit3
    source $AUTOSUBMIT_VENV/bin/activate

    if [[ -n "$(autosubmit 2>&1 /dev/null)" ]]; then 
        echo "Loaded!"
    else
        echo "testing_suite_wrapper.sh: Error: Could not load Autosubmit from venv $AUTOSUBMIT_VENV"    
    fi

}

function setup_environment() {
    #
    # This function sets the api, the mamba environment with testing suite, and
    # the autosubmit environment
    #
    
    load_autosubmit
    load_autosubmit_api
    load_testing_suite 

}

function unload_environment() {
    stop_autosubmit_api
}

# run locally the testing suite

case "$1" in 
    create)
        setup_environment
        testing_suite --skip_folder_check --create 
        unload_environment
        ;;
    
    status) # check the status of the jobs
        setup_environment
        testing_suite --skip_folder_check --status
        unload_environment
        ;;
    
    run) # the testing suite 
        if [[ -z "$2" ]]; then
            echo "testing_suite_wrapper.sh: Error: You have to specify the number of interactions"
            exit 
        fi 
        setup_environment
        for (( i=0 ; i < $2; ++i )); do
            print "Running iteration $i" 
            echo "testing_suite --skip_folder_check --create" 
            echo "testing_suite --skip_folder_check --skip_folder_check --run" 
            echo "testing_suite --skip_folder_check --status"
        done        
        unload_environment
        ;;

    status) # check the status of the jobs
        setup_environment
        testing_suite --status
        unload_environment
    ;;

    "") # just output experiments and description
        setup_environment
        testing_suite
        unload_environment
    ;;
    
    *)
        echo "testing_suite_wrapper.sh: Error: Incorrect command. You have to choose between 'run', 'status', or 'create'"
    ;;
esac 

